import 'package:stateful_provider/model/dart_error.dart';
import 'package:uuid/uuid.dart';
import 'task_provider.dart';

class ProcessStatusProvider extends TaskProvider {
  factory ProcessStatusProvider() => _singleton;

  static final ProcessStatusProvider _singleton =
      ProcessStatusProvider._internal();

  ProcessStatusProvider._internal();

  List<DartError> _exceptions = [];

  void add(Uuid uuid) {
    processes.add(uuid);
    _rebuildIfNeeded();
  }

  void complete(Uuid uuid) {
    processes.remove(uuid);
    _rebuildIfNeeded();
  }

  bool _isProcessing = false;

  void _rebuildIfNeeded() {
    final current = isProcessing;
    if (current == _isProcessing) return;
    _isProcessing = current;
    notifyListeners();
  }

  bool _handlingError = false;

  bool get hasError => _exceptions.isNotEmpty;

  List<DartError> get errors => _exceptions;

  DartError currentError() {
    _handlingError = true;
    return _exceptions.first;
  }

  void catchError(dynamic error) {
    _exceptions.add(DartError(error));
    if (_handlingError) return;
    notifyListeners();
  }

  void consumeError(DartError error) {
    _exceptions.removeWhere((element) => element.uuid == error.uuid);
    _handlingError = false;
    notifyListeners();
  }
}
